from juungle.auth import Auth


class User(Auth):
    def __init__(self, login_user=None, login_pass=None):
        Auth.__init__(self, login_user, login_pass)
        self.user_id = None
        self.email = None
        self.deposit_address = None
        self.bch_balance = None

        self._load_user()

    def _load_user(self):
        response = self.call_get_json("/user/details", {}, True)
        j_response = response.json()
        self.user_id = j_response['id']
        self.email = j_response['email']
        self.deposit_address = j_response['depositAddress']
        self.bch_balance = j_response['bchBalance']

    def withdraw_bch(self, to_address):
        data = {
            "toAddress": to_address,
            "password": self.login_pass
        }
        response = self.call_post('user/withdraw_bch', data, True)
        print(response.json()['txid'])



"""Auth module."""
import configparser
import os
import time

import requests

from juungle.exception import (CommandFailed, FailedRequest, NoLoginProvided,
                               NoPasswordProvided, TooManyRequests)
from juungle.rate_limiter import Limiter

URL_API = "https://www.juungle.net/api/v1"


class Auth():
    """Auth Class."""

    def __init__(self, login_user=None, login_pass=None):
        """init."""

        self.login_user = login_user
        self.login_pass = login_pass

        if not self.login_user or not self.login_pass:
            if os.path.exists('user-config.ini'):
                config = configparser.ConfigParser()
                config.read('user-config.ini')
                self.login_user = config['DEFAULT']['LOGIN_USERNAME']
                self.login_pass = config['DEFAULT']['LOGIN_PASSWORD']

        if not self.login_user:
            raise NoLoginProvided

        if not self.login_pass:
            raise NoPasswordProvided

        self._limiter = None

    def _get_token(self):
        data = {
            "email": self.login_user,
            "password": self.login_pass
        }

        url = "{}/{}".format(URL_API, 'user/login')
        if self._limiter:
            if self._limiter.rate_limit_remaining > 1:
                response = requests.post(url, json=data, headers={
                    'Content-Type': 'application/json'})
                self._limiter.update(response.headers)
            else:
                time.sleep(self._limiter.rate_limit_reset)
                response = requests.post(url, json=data, headers={
                    'Content-Type': 'application/json'})
                self._limiter.update(response.headers)
        else:
            response = requests.post(
                url, json=data, headers={'Content-Type': 'application/json'})
            self._limiter = Limiter(response.headers)

        if response.status_code == 200:
            r_json = response.json()
            if not r_json['success']:
                raise CommandFailed('Login failed: {}'.format(
                    response.content))
            return r_json['jwtToken']

        if response.status_code == 429:
            raise TooManyRequests

        if response.status_code != 200:
            raise CommandFailed(response.json()["message"])

    def _check_response(self, response):
        if response.status_code != 200:
            if response.status_code == 429:
                raise TooManyRequests
            raise FailedRequest('Request failed: {}'.format(response.content))

        j_response = response.json()
        if not j_response['success']:
            raise CommandFailed('Command failed: {}'.format(response.content))

    def call_post(self, url, data, with_token=False,
                  headers={'Content-Type': 'application/json'}):
        """Methods post."""

        if with_token:
            headers['X-Access-Token'] = self._get_token()

        url = "{}/{}".format(URL_API, url)

        if self._limiter:
            if self._limiter.rate_limit_remaining > 1:
                response = requests.post(url, json=data, headers=headers)
                self._limiter.update(response.headers)
            else:
                time.sleep(self._limiter.rate_limit_reset)
                response = requests.post(url, json=data, headers=headers)
                self._limiter.update(response.headers)
        else:
            response = requests.post(url, json=data, headers=headers)
            self._limiter = Limiter(response.headers)

        self._check_response(response)

        return response

    def call_get_json(self, url, data, with_token=False,
                      headers={'Content-Type': 'application/json'}):

        if with_token:
            headers['X-Access-Token'] = self._get_token()

        url = "{}/{}".format(URL_API, url)

        if self._limiter:
            if self._limiter.rate_limit_remaining > 1:
                response = requests.get(url, json=data, headers=headers)
                self._limiter.update(response.headers)
            else:
                print('Requests are too fast. Waiting a bit....')
                time.sleep(self._limiter.rate_limit_reset)
                print('Ok all good.. continue')
                response = requests.get(url, json=data, headers=headers)
                self._limiter.update(response.headers)
        else:
            response = requests.get(url, json=data, headers=headers)
            self._limiter = Limiter(response.headers)

        return response

    def call_get_query(self, url, data):
        url = "{}/{}".format(URL_API, url)
        response = requests.get(url, params=data)
        self._check_response(response)
        return response




class Limiter:
    def __init__(self, headers):
        self.rate_limit_remaining = int(headers['RateLimit-Remaining'])
        self.rate_limit_reset = int(headers['RateLimit-Reset'])

    def update(self, headers):
        self.rate_limit_remaining = int(headers['RateLimit-Remaining'])
        self.rate_limit_reset = int(headers['RateLimit-Reset'])



from collections import defaultdict

from juungle.auth import Auth
from juungle.tokens import TOKENS_IDS
from juungle.user import User


class NFTs(Auth):
    def __init__(self, login_user=None, login_pass=None):
        Auth.__init__(self, login_user, login_pass)
        self.clear()

    def add_nft(self, nft_info):
        nft = NFT(nft_info, self.login_user, self.login_pass)
        self.list_nfts.append(nft)
        self.group_ids[nft.token_id].append(nft)

    def get_nfts(self):
        response = self.call_get_query('/nfts', self._search_options)
        l_nfts = response.json()
        for nft in l_nfts['nfts']:
            self.add_nft(nft)

    def clear(self):
        self.list_nfts = list()
        self._search_options = {
            'offset': 0,
            'limit': 10000
        }
        self.group_ids = defaultdict(list)

    @property
    def token_group(self):
        return self._search_options['groupTokenId']

    @token_group.setter
    def token_group(self, value):
        self._search_options['groupTokenId'] = TOKENS_IDS[value] \
            if value in TOKENS_IDS else value

    @property
    def offset(self):
        return self._search_options['offset']

    @offset.setter
    def offset(self, value):
        self._search_options['offsite'] = value

    @property
    def limit(self):
        return self._search_options['limit']

    @limit.setter
    def limit(self, value):
        # limit has to be less than 10 000
        if value < 10000:
            self._search_options['limit'] = value

    @property
    def purchased(self):
        if 'purchaseTxidSet' in self._search_options:
            return self._search_options['purchaseTxidSet']

    @purchased.setter
    def purchased(self, value):
        if isinstance(value, bool):
            self._search_options['purchaseTxidSet'] = str(value).lower()
        else:
            raise ValueError('Purchased must be True or False')

    @property
    def deposited(self):
        if 'depositTxidSet' in self._search_options:
            return self._search_options['depositTxidSet']

    @deposited.setter
    def deposited(self, value):
        if isinstance(value, bool):
            self._search_options['depositTxidSet'] = str(value).lower()
        else:
            raise ValueError('Deposited must be True or False')

    @property
    def available_to_buy(self):
        if 'priceSatoshisSet' in self._search_options:
            return self._search_options['priceSatoshisSet']

    @available_to_buy.setter
    def available_to_buy(self, value):
        if isinstance(value, bool):
            self._search_options['priceSatoshisSet'] = str(value).lower()
            self._search_options['depositTxidSet'] = str(value).lower()
            self._search_options['withdrawTxid'] = str(not value).lower()
            self._search_options['purchaseTxid'] = str(not value).lower()
        else:
            raise ValueError('Available to buy must be True or False')

    def get_my_nfts(self):
        self.clear()
        user = User(self.login_user, self.login_pass)
        self._search_options['userId'] = user.user_id
        # Wait a little bit before trying to query again
        self.get_nfts()


class NFT(Auth):
    def __init__(self, nft_info, login_user=None, login_pass=None):
        self.nft_id = nft_info["id"]
        self.user_id = nft_info["userId"]
        self.deposit_txid = nft_info["depositTxid"]
        self.withdraw_txid = nft_info["withdrawTxid"]
        self.purchase_txid = nft_info["purchaseTxid"]
        self.token_id = nft_info["tokenId"]
        self.token_name = nft_info["tokenName"]
        self.token_symbol = nft_info["tokenSymbol"]
        self.group_tokenid = nft_info["groupTokenId"]
        self.price_satoshis = nft_info["priceSatoshis"]
        if nft_info["priceSatoshis"]:
            self.price_bch = nft_info["priceSatoshis"] / 100000000
        else:
            self.price_bch = nft_info["priceSatoshis"]
        self.ts = nft_info["ts"]
        self.purchase_hold = nft_info["purchaseHold"]
        self.buy_price = None
        self.buy_address = None

        self.image = 'https://juungle.net/api/v1/nfts/icon/{}/{}'.format(
            self.group_tokenid, self.token_id
        )
        Auth.__init__(self, login_user, login_pass)

    @property
    def name(self):
        return self.token_name

    @property
    def is_purchased(self):
        return bool(self.purchase_txid)

    @property
    def is_sold(self):
        return self.is_purchased

    def _convert_bch_to_sats(self, bch):
        return bch * 100000000

    @property
    def is_for_sale(self):
        return self.price_satoshis and self.deposit_txid and \
            not self.withdraw_txid and not self.purchase_txid

    def buy(self, to_address):
        if not to_address:
            raise ValueError('BCH Address must be inform!')

        data = {
            "nftId": self.nft_id,
            "address": to_address
        }

        response = self.call_post('nfts/create_purchase_hold', data,
                                  True).json()

        self.buy_price = response['priceSatoshis']
        self.buy_address = response['address']

    def set_price(self, sats=None, bch=None):

        if not sats and not bch:
            raise ValueError('Value must be set as satoshis or BCH')

        price = sats
        if bch:
            price = self._convert_bch_to_sats(bch)

        data = {
            "nftId": self.nft_id,
            "priceSatoshis": price
        }

        self.call_post('user/nfts/set_price', data, True)

    def cancel_sale(self):
        data = {
            "nftId": self.nft_id,
        }

        self.call_post('user/nfts/cancel_sale', data, True)



"""Setup module."""
import setuptools

with open("README.md", "r", encoding="utf-8") as f_file:
    long_description = f_file.read()

extras = {
    "test": [
        "pytest >=2.7.3",
    ]
}


setuptools.setup(
    name="juungle",
    version="0.6.1",
    author="Eduardo Elias",
    author_email="camponez@gmail.com",
    description="Juungle python package for juungle.net",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    keywords="juungle.net api nfts",
    url="https://github.com/Juungle/juungle-python",
    project_urls={
        "Bug Tracker": "https://github.com/Juungle/juungle-python/issues",
        "Source Code": "https://github.com/Juungle/juungle-python",
    },
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
    ],
    install_requires=[
        'requests'
    ],
    extras_require=extras,
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)



class NoLoginProvided(Exception):
    pass


class NoPasswordProvided(Exception):
    pass


class LoginFailed(Exception):
    pass


class CommandFailed(Exception):
    pass


class FailedRequest(Exception):
    pass


class TooManyRequests(Exception):
    pass
